use crate::schema::users;
use diesel::prelude::*;

/// User ID holder
#[derive(Debug, Clone, Copy, serde::Serialize, serde::Deserialize)]
pub struct UserID(pub i32);

#[derive(Queryable, Debug, serde::Serialize)]
pub struct User {
    pub id: i32,
    pub name: String,
    pub email: String,
    #[serde(skip_serializing)]
    pub password: Option<String>,
    pub time_create: i64,
    #[serde(skip_serializing)]
    pub reset_password_token: Option<String>,
    #[serde(skip_serializing)]
    pub time_gen_reset_token: i64,
    #[serde(skip_serializing)]
    pub delete_account_token: Option<String>,
    #[serde(skip_serializing)]
    pub time_gen_delete_account_token: i64,
    pub time_activate: i64,
    pub active: bool,
    pub admin: bool,
}

impl User {
    pub fn id(&self) -> UserID {
        UserID(self.id)
    }

    pub fn check_password(&self, password: &str) -> bool {
        self.password
            .as_deref()
            .map(|hash| {
                bcrypt::verify(password, hash).unwrap_or_else(|e| {
                    log::error!("Failed to validate password! {}", e);
                    false
                })
            })
            .unwrap_or(false)
    }
}

#[derive(Insertable)]
#[diesel(table_name = users)]
pub struct NewUser<'a> {
    pub name: &'a str,
    pub email: &'a str,
    pub time_create: i64,
}
